@@ -163,6 +163,8 @@ static HANDLE_FUNC (handle_upstream);
 static HANDLE_FUNC (handle_upstream_no);
 #endif
 
+static void config_free_regex (void);
+
 /*
  * This macro can be used to make standard directives in the form:
  *   directive arguments [arguments ...]
@@ -317,7 +319,8 @@ static void free_config (struct config_s *conf)
  *
  * Returns 0 on success; negative upon failure.
  */
-static int config_compile (void)
+int
+config_compile_regex (void)
 {
         unsigned int i, r;
 
@@ -335,9 +338,30 @@ static int config_compile (void)
                 if (r)
                         return r;
         }
+
+        atexit (config_free_regex);
+
         return 0;
 }
 
+/*
+ * Frees pre-compiled regular expressions used by the configuration
+ * file. This function is registered to be automatically called at exit.
+ */
+static void
+config_free_regex (void)
+{
+        unsigned int i;
+
+        for (i = 0; i < ndirectives; i++) {
+                if (directives[i].cre) {
+                        regfree (directives[i].cre);
+                        safefree (directives[i].cre);
+                        directives[i].cre = NULL;
+                }
+        }
+}
+
 /*
  * Attempt to match the supplied line with any of the configuration
  * regexes defined above.  If a match is found, call the handler
@@ -397,7 +421,7 @@ static int load_config_file (const char *config_fname, struct config_s *conf)
                 goto done;
         }
 
-        if (config_compile () || config_parse (conf, config_file)) {
+        if (config_parse (conf, config_file)) {
                 fprintf (stderr, "Unable to parse config file. "
                          "Not starting.\n");
                 goto done;
@@ -617,8 +641,8 @@ set_bool_arg (unsigned int *var, const char *line, regmatch_t * match)
         return 0;
 }
 
-static unsigned long int
-get_int_arg (const char *line, regmatch_t * match)
+static unsigned long
+get_long_arg (const char *line, regmatch_t * match)
 {
         assert (line);
         assert (match && match->rm_so != -1);
@@ -627,13 +651,13 @@ get_int_arg (const char *line, regmatch_t * match)
 }
 
 static int
-set_int_arg (unsigned long int *var, const char *line, regmatch_t * match)
+set_int_arg (unsigned int *var, const char *line, regmatch_t * match)
 {
         assert (var);
         assert (line);
         assert (match);
 
-        *var = get_int_arg (line, match);
+        *var = (unsigned int) get_long_arg (line, match);
         return 0;
 }
 
@@ -755,49 +779,50 @@ static HANDLE_FUNC (handle_bindsame)
 
 static HANDLE_FUNC (handle_port)
 {
-        return set_int_arg ((unsigned long int *) &conf->port, line, &match[2]);
+        return set_int_arg (&conf->port, line, &match[2]);
 }
 
 static HANDLE_FUNC (handle_maxclients)
 {
-        child_configure (CHILD_MAXCLIENTS, get_int_arg (line, &match[2]));
+        child_configure (CHILD_MAXCLIENTS, get_long_arg (line, &match[2]));
         return 0;
 }
 
 static HANDLE_FUNC (handle_maxspareservers)
 {
-        child_configure (CHILD_MAXSPARESERVERS, get_int_arg (line, &match[2]));
+        child_configure (CHILD_MAXSPARESERVERS,
+                         get_long_arg (line, &match[2]));
         return 0;
 }
 
 static HANDLE_FUNC (handle_minspareservers)
 {
-        child_configure (CHILD_MINSPARESERVERS, get_int_arg (line, &match[2]));
+        child_configure (CHILD_MINSPARESERVERS,
+                         get_long_arg (line, &match[2]));
         return 0;
 }
 
 static HANDLE_FUNC (handle_startservers)
 {
-        child_configure (CHILD_STARTSERVERS, get_int_arg (line, &match[2]));
+        child_configure (CHILD_STARTSERVERS, get_long_arg (line, &match[2]));
         return 0;
 }
 
 static HANDLE_FUNC (handle_maxrequestsperchild)
 {
         child_configure (CHILD_MAXREQUESTSPERCHILD,
-                         get_int_arg (line, &match[2]));
+                         get_long_arg (line, &match[2]));
         return 0;
 }
 
 static HANDLE_FUNC (handle_timeout)
 {
-        return set_int_arg ((unsigned long int *) &conf->idletimeout, line,
-                            &match[2]);
+        return set_int_arg (&conf->idletimeout, line, &match[2]);
 }
 
 static HANDLE_FUNC (handle_connectport)
 {
-        add_connect_port_allowed (get_int_arg (line, &match[2]),
+        add_connect_port_allowed (get_long_arg (line, &match[2]),
                                   &conf->connect_ports);
         return 0;
 }
@@ -866,7 +891,7 @@ static HANDLE_FUNC (handle_errorfile)
          * present.  This is why the "string" is located at
          * match[4] (rather than the more intuitive match[3].
          */
-        unsigned long int err = get_int_arg (line, &match[2]);
+        unsigned long int err = get_long_arg (line, &match[2]);
         char *page = get_string_arg (line, &match[4]);
 
         add_new_errorpage (page, err);
@@ -1019,7 +1044,7 @@ static HANDLE_FUNC (handle_upstream)
         ip = get_string_arg (line, &match[2]);
         if (!ip)
                 return -1;
-        port = (int) get_int_arg (line, &match[7]);
+        port = (int) get_long_arg (line, &match[7]);
 
         if (match[9].rm_so != -1) {
                 domain = get_string_arg (line, &match[9]);
